@@ -1,15 +1,13 @@
 use std::collections::HashSet;
 
-use js_sys::Array;
-use wasm_bindgen::prelude::*;
 use web_sys::{IdbDatabase, IdbObjectStoreParameters, IdbOpenDbRequest};
 
-use crate::{Error, Index, Result};
+use crate::{key_path::KeyPath, Error, Index, Result};
 
 /// An object store builder.
 pub struct ObjectStore {
     pub(crate) name: String,
-    pub(crate) key_path: Vec<String>,
+    pub(crate) key_path: Option<KeyPath>,
     pub(crate) auto_increment: Option<bool>,
     pub(crate) indexes: Vec<Index>,
 }
@@ -19,7 +17,7 @@ impl ObjectStore {
     pub fn new(name: &str) -> Self {
         Self {
             name: name.to_owned(),
-            key_path: Vec::new(),
+            key_path: None,
             auto_increment: None,
             indexes: Vec::new(),
         }
@@ -27,13 +25,13 @@ impl ObjectStore {
 
     /// Specify key path for the object store
     pub fn key_path(mut self, key_path: &str) -> Self {
-        self.key_path = vec![key_path.to_owned()];
+        self.key_path = Some(KeyPath::new_str(key_path));
         self
     }
 
     /// Specify key path array for the object store
     pub fn key_path_array<'a>(mut self, key_path_array: impl IntoIterator<Item = &'a str>) -> Self {
-        self.key_path = key_path_array.into_iter().map(ToOwned::to_owned).collect();
+        self.key_path = Some(KeyPath::new_array(key_path_array));
         self
     }
 
@@ -73,23 +71,8 @@ impl ObjectStore {
                 params.auto_increment(auto_increment);
             }
 
-            match self.key_path.len() {
-                0 => {
-                    params.key_path(None);
-                }
-                1 => {
-                    params.key_path(Some(&(&self.key_path[0]).into()));
-                }
-                _ => {
-                    params.key_path(Some(
-                        &self
-                            .key_path
-                            .into_iter()
-                            .map(|key| JsValue::from_str(&key))
-                            .collect::<Array>()
-                            .into(),
-                    ));
-                }
+            if let Some(key_path) = self.key_path {
+                params.key_path(Some(&key_path.into()));
             }
 
             idb.create_object_store_with_optional_parameters(&self.name, &params)
